jest.mock('fs')
jest.mock('child_process')
jest.mock('os')
const md5 = require('md5')
const { resolvers, typeDefs, __test__ } = require('../../src/interfaces')
const child_process = require('child_process')
child_process.addCRPair({
  input: 'options',
  output: "[{'option1':'value1'},{'option2':'value2'}]"
})

describe('interfaces module', () => {
  it('exports resolvers and typeDefs', () => {
    expect(resolvers).toBeDefined()
    expect(typeDefs).toBeDefined()
  })
  it('resolves queries and mutations', () => {
    expect(resolvers).toHaveProperty('Query'),
    expect(resolvers).toHaveProperty('Mutation')
  })

  it('finds workers', async () => {
    const { findWorkers } = __test__
    await expect(findWorkers('/')).resolves.toEqual([])
    await expect(findWorkers('python_workers')).resolves.toEqual([
      'test1_worker.py',
      'test2_worker.py'
    ])
    await expect(findWorkers()).rejects.toThrow(
      'Directory argument must be a string.'
    )
    await expect(findWorkers('/DOESNTEXIST')).rejects.toThrow(
      `ENOENT: no such file or directory, scandir '/DOESNTEXIST'`
    )
  })

  it('finds options', async () => {
    const { getOptions } = __test__
    const options = [{ option1: 'value1' }]
    const send = jest.fn()
    const workerProcess = { send }

    // 1. Check the use case
    send.mockReturnValueOnce(
      new Promise((resolve, reject) => {
        resolve({ data: options })
      })
    )
    await expect(getOptions(workerProcess)).resolves.toEqual(options)

    // 2. Check empty option case
    expect(workerProcess.send).toHaveBeenCalledWith({ type: 'options' })
    send.mockReturnValueOnce(
      new Promise((resolve, reject) => {
        resolve({ data: [] })
      })
    )
    await expect(getOptions(workerProcess)).resolves.toEqual([])

    // 3. Check error case
    expect(workerProcess.send).toHaveBeenCalledWith({ type: 'options' })
    send.mockReturnValueOnce(
      new Promise((resolve, reject) => {
        resolve({ error: 'Timeout' })
      })
    )
    await expect(getOptions(workerProcess)).rejects.toThrow('Timeout')

    // 4. Throw error if workerProcess doesn't have a "send" property.
    await expect(getOptions()).rejects.toThrow(
      'workerProcess not configured properly.'
    )
  })

  it('generates interface', async () => {
    const { createInterface } = __test__
    await expect(createInterface()).rejects.toThrow(
      'Directory argument must be a string.'
    )
    await expect(
      createInterface('python_workers', 'testXXX_worker.py')
    ).rejects.toThrow(
      `ENOENT: no such file or directory, scandir 'python_workers/testXXX_worker.py'`
    )
    await expect(
      createInterface('python_workers', 'test1_worker.py')
    ).resolves.toMatchObject({
      id: md5(`test1python_workers/test1_worker.py1980-03-18T03:15:00.000Z`),
      interfaceName: 'test1',
      options: [],
      workerProcess: {
        error: []
      },
      dini: 12
    })
  })

  const { Query, Mutation, Interface, Connection } = resolvers
  xit('finds interfaces', async () => {
    const resp = await Query.interfaces()
    expect(resp).toBe([])
  })
})
